import bpy
import gpu
import blf
import bpy_extras
import math
from gpu_extras.batch import batch_for_shader
from mathutils import Vector, Matrix
from bpy.types import Operator, Panel
from bpy.props import PointerProperty
from bpy_extras import view3d_utils
from .lighting_gadgets_utils import update_light_transform, calculate_hit_distance, find_closest_surface_point, update_light_precise_transform

def draw_callback_px(self, context):
    """Draw viewport border and mode hint text"""
    if not self.is_active or context.area != self._active_area:
        return
    
    # 获取首选项
    prefs = context.preferences.addons[__package__].preferences
        
    # 设置颜色和文本
    if self._precise_mode:
        base_color = prefs.precise_color  # 精确模式颜色
        mode_text = "Precise Control Mode（Press Space to Switch Modes）"
    else:
        base_color = prefs.reflection_color  # 反射模式颜色
        mode_text = "Reflection Mode（Press Space to Switch Modes）"
    
    # 创建带透明度的颜色
    border_color = (*base_color, prefs.border_opacity)
    text_color = (*base_color, prefs.text_opacity)
    
    # 获取区域尺寸
    width = context.area.width
    height = context.area.height
    
    # 创建着色器
    shader = gpu.shader.from_builtin('UNIFORM_COLOR')
    
    # 设置OpenGL状态
    gpu.state.blend_set('ALPHA')  # 启用alpha混合
    gpu.state.line_width_set(prefs.border_thickness)
    
    # 定义边框顶点 - 分别绘制每条边以使用不同的宽度
    vertices_left = ((0, 0), (0, height))
    vertices_top = ((0, height), (width, height))
    vertices_right = ((width, height), (width, 0))
    vertices_bottom = ((width, 0), (0, 0))
    
    # 绘制左边和底边
    batch = batch_for_shader(shader, 'LINES', {"pos": vertices_left + vertices_bottom})
    shader.bind()
    shader.uniform_float("color", border_color)
    gpu.state.line_width_set(prefs.border_thickness * 1.0)
    batch.draw(shader)
    
    # 绘制右边和顶边（更粗）
    batch = batch_for_shader(shader, 'LINES', {"pos": vertices_right + vertices_top})
    shader.bind()
    shader.uniform_float("color", border_color)
    gpu.state.line_width_set(prefs.border_thickness * 1.2)  # 稍微粗一些
    batch.draw(shader)
    
    # 如果有命中点，绘制一个小白点
    if hasattr(self, '_hit_point') and self._hit_point is not None:
        # 获取命中点的2D坐标
        region = context.region
        rv3d = context.region_data
        hit_2d = bpy_extras.view3d_utils.location_3d_to_region_2d(region, rv3d, self._hit_point)
        
        if hit_2d:
            # 定义点的顶点
            radius = 4  # 点的大小
            segments = 16  # 圆形段数
            center_x, center_y = hit_2d
            
            # 创建圆形顶点
            import math
            vertices = []
            for i in range(segments):
                angle = 2 * math.pi * i / segments
                x = center_x + radius * math.cos(angle)
                y = center_y + radius * math.sin(angle)
                vertices.append((x, y))
            
            # 绘制白色填充
            batch = batch_for_shader(shader, 'TRI_FAN', {"pos": [(center_x, center_y)] + vertices})
            shader.bind()
            shader.uniform_float("color", (1.0, 1.0, 1.0, prefs.text_opacity))
            batch.draw(shader)
            
            # 绘制黑色边框
            vertices.append(vertices[0])  # 闭合路径
            batch = batch_for_shader(shader, 'LINE_STRIP', {"pos": vertices})
            shader.bind()
            shader.uniform_float("color", (0.0, 0.0, 0.0, prefs.text_opacity))
            gpu.state.line_width_set(1.0)
            batch.draw(shader)
    
    # 绘制文本
    font_id = 0  # 内置字体
    
    # 设置字体大小
    blf.size(font_id, prefs.text_size)
    
    # 获取文本尺寸
    text_width, text_height = blf.dimensions(font_id, mode_text)
    
    # 计算文本位置（左下角，带边距）
    margin = 30
    pos_x = margin
    pos_y = margin + text_height
    
    # 绘制文本轮廓（使用多个偏移）
    offsets = ((-1, -1), (-1, 1), (1, -1), (1, 1),
              (-1, 0), (1, 0), (0, -1), (0, 1))
    
    # 绘制轮廓
    blf.color(font_id, 0.0, 0.0, 0.0, prefs.text_opacity)
    for offset_x, offset_y in offsets:
        blf.position(font_id, pos_x + offset_x, pos_y + offset_y, 0)
        blf.draw(font_id, mode_text)
    
    # 绘制文本
    blf.color(font_id, *text_color)
    blf.position(font_id, pos_x, pos_y, 0)
    blf.draw(font_id, mode_text)
    
    # 重置OpenGL状态
    gpu.state.blend_set('NONE')  # 禁用alpha混合
    gpu.state.line_width_set(1.0)

class LIGHTING_GADGETS_OT_REFLECTION_MODE(Operator):
    bl_idname = "lighting_gadgets.reflection_mode"
    bl_label = "Reflection Mode"
    bl_description = ("Enable Reflection Mode (Shortcut: Shift + R)\n"
                     "Hold Shift to speed up in standard mode\n"
                     "Press Space to enter precise mode, hold Shift to slow down in precise mode")
    bl_options = {'REGISTER', 'UNDO'}

    _timer = None
    is_active = False
    _initial_transform = None
    _initial_distance = None
    _last_hit_location = None
    _last_hit_normal = None
    _draw_handle = None
    _active_area = None  # Store the area where reflection mode was triggered
    _precise_mode = False  # Whether in precise control mode
    _last_mouse_pos = None  # Mouse position for precise control mode
    _hit_point = None  # Hit point
    _last_hit_distance = None  # Last hit distance
    _initial_location = None
    _initial_rotation = None

    @classmethod
    def poll(cls, context):
        return context.active_object and context.active_object.type == 'LIGHT'

    def modal(self, context, event):
        if not context.active_object or context.active_object.type != 'LIGHT':
            self.cleanup(context, cancel=True)
            return {'CANCELLED'}

        # Only trigger redraw in active area
        if context.area == self._active_area:
            context.area.tag_redraw()

        # Handle space key for precise control mode toggle
        if event.type == 'SPACE' and event.value == 'PRESS':
            if self._last_hit_location is not None:  # Only toggle if we have a hit point
                self._precise_mode = not self._precise_mode
                self._last_mouse_pos = (event.mouse_region_x, event.mouse_region_y)
                # Force redraw to update border color
                if context.area:
                    context.area.tag_redraw()
                return {'RUNNING_MODAL'}
            return {'RUNNING_MODAL'}  # Keep current mode if no hit point

        if event.type == 'TIMER':
            light_obj = context.active_object
            
            if self._precise_mode:
                # Precise control mode
                if self._last_mouse_pos is None:
                    self._last_mouse_pos = (event.mouse_region_x, event.mouse_region_y)
                    return {'RUNNING_MODAL'}
                
                # Get current mouse position
                mouse_pos = (event.mouse_region_x, event.mouse_region_y)
                
                # Perform light transformation
                update_light_precise_transform(
                    context,
                    light_obj,
                    self._last_hit_location,
                    self._last_hit_normal,
                    mouse_pos,
                    self._last_mouse_pos,
                    event.shift
                )
                
                # Update last mouse position
                self._last_mouse_pos = mouse_pos
                
                context.view_layer.update()
                
            else:
                # Standard reflection mode
                coord = (event.mouse_region_x, event.mouse_region_y)
                region = context.region
                rv3d = context.region_data
                
                # Get ray origin and direction
                ray_origin = view3d_utils.region_2d_to_origin_3d(region, rv3d, coord)
                ray_direction = view3d_utils.region_2d_to_vector_3d(region, rv3d, coord)
                
                # Perform ray casting
                result = context.scene.ray_cast(
                    depsgraph=context.view_layer.depsgraph,
                    origin=ray_origin,
                    direction=ray_direction
                )
                
                if result[0]:  # If ray hits an object
                    hit_location = result[1]
                    hit_normal = result[2]
                    
                    # Set movement speed
                    if self._last_hit_location is None:
                        self._last_hit_location = hit_location.copy()
                        self._last_hit_normal = hit_normal.copy()
                    
                    # Choose interpolation factor based on Shift key
                    lerp_factor = 0.8 if event.shift else 0.15  # Shift speeds up movement
                    
                    # Interpolate between current and new positions
                    smoothed_location = self._last_hit_location.lerp(hit_location, lerp_factor)
                    smoothed_normal = self._last_hit_normal.lerp(hit_normal, lerp_factor)
                    
                    # Update last position and normal
                    self._last_hit_location = smoothed_location
                    self._last_hit_normal = smoothed_normal
                    
                    # Apply reflection calculation
                    update_light_transform(
                        context.region_data,
                        light_obj,
                        smoothed_location,
                        smoothed_normal,
                        self._initial_distance
                    )
                    
                    context.view_layer.update()
                    
                    # Update hit point
                    self._hit_point = hit_location
                    
        elif event.type == 'LEFTMOUSE' and event.value == 'PRESS':
            # 确认时保存落点数据
            if self._last_hit_location is not None:
                light = context.active_object
                # 保存落点数据到自定义属性
                light["last_hit_location"] = self._last_hit_location.copy()
                light["last_hit_distance"] = self._last_hit_distance
                light["last_hit_normal"] = self._last_hit_normal.copy()
                
                # 清理绘制
                if self._timer is not None:
                    context.window_manager.event_timer_remove(self._timer)
                bpy.types.SpaceView3D.draw_handler_remove(self._draw_handle, 'WINDOW')
                context.area.tag_redraw()
                
                return {'FINISHED'}
            return {'CANCELLED'}
            
        elif event.type in {'RIGHTMOUSE', 'ESC'}:
            # 取消时恢复初始状态，不保存落点
            light = context.active_object
            light.location = self._initial_location
            light.rotation_euler = self._initial_rotation
            
            # 清理
            if self._timer is not None:
                context.window_manager.event_timer_remove(self._timer)
            bpy.types.SpaceView3D.draw_handler_remove(self._draw_handle, 'WINDOW')
            context.area.tag_redraw()
            return {'CANCELLED'}

        return {'RUNNING_MODAL'}

    def invoke(self, context, event):
        if context.area.type != 'VIEW_3D':
            self.report({'WARNING'}, "Use this tool in the 3D view")
            return {'CANCELLED'}
            
        if not self.poll(context):
            self.report({'ERROR'}, "Select a light object first")
            return {'CANCELLED'}
        
        if self.is_active:
            self.cleanup(context, cancel=True)
            return {'CANCELLED'}
        
        light_obj = context.active_object
        
        # Save initial state
        self._initial_location = light_obj.location.copy()
        self._initial_rotation = light_obj.rotation_euler.copy()
        
        # Try to find the closest surface point to the current light direction
        hit_location, hit_normal, distance = find_closest_surface_point(context, light_obj)
        
        if hit_location is not None:
            # If a surface point is found, use the actual distance
            self._initial_distance = distance
        else:
            # If no surface point is found, use the distance to the world center
            self._initial_distance = light_obj.location.length
        
        # Load last hit point data
        if not self.load_last_hit_data(light_obj):
            # If no history data, use the current hit point
            if hit_location is not None:
                self._hit_point = hit_location
                self._last_hit_location = hit_location
                self._last_hit_normal = hit_normal
                self._last_hit_distance = distance
        
        # Set timer
        wm = context.window_manager
        self._timer = wm.event_timer_add(0.016, window=context.window)
        
        # Store current active area
        self._active_area = context.area
        self.is_active = True
        
        # Add drawing callback
        args = (self, context)
        self._draw_handle = bpy.types.SpaceView3D.draw_handler_add(
            draw_callback_px, args, 'WINDOW', 'POST_PIXEL'
        )
        
        context.area.tag_redraw()
        
        wm.modal_handler_add(self)
        return {'RUNNING_MODAL'}

    def load_last_hit_data(self, light):
        """Load last hit point data from the light object"""
        try:
            if "last_hit_location" in light and "last_hit_normal" in light and "last_hit_distance" in light:
                self._last_hit_location = Vector(light["last_hit_location"])
                self._last_hit_normal = Vector(light["last_hit_normal"])
                self._last_hit_distance = float(light["last_hit_distance"])
                self._hit_point = self._last_hit_location
                return True
        except:
            pass
        return False

    def cleanup(self, context, cancel):
        if self._timer:
            context.window_manager.event_timer_remove(self._timer)
            self._timer = None
            
        if self._draw_handle:
            bpy.types.SpaceView3D.draw_handler_remove(self._draw_handle, 'WINDOW')
            self._draw_handle = None
            
        if cancel:
            # Restore initial state
            context.active_object.location = self._initial_location
            context.active_object.rotation_euler = self._initial_rotation
            
        self.is_active = False
        context.area.tag_redraw()

class LIGHTING_GADGETS_PT_QUICK_PANEL(Panel):
    bl_label = "Light Settings"
    bl_idname = "LIGHTING_GADGETS_PT_quick_panel"
    bl_space_type = 'VIEW_3D'
    bl_region_type = 'WINDOW'
    bl_ui_units_x = 12

    @classmethod
    def poll(cls, context):
        return context.active_object and context.active_object.type == 'LIGHT'

    def find_connected_nodes(self, output_node, connected_nodes=None):
        """递归查找与输出节点相连的所有节点"""
        if connected_nodes is None:
            connected_nodes = set()
            
        # 如果节点已经处理过，直接返回
        if output_node in connected_nodes:
            return connected_nodes
            
        connected_nodes.add(output_node)
        
        # 遍历所有输入端口
        for input in output_node.inputs:
            # 检查输入端口是否有连接
            if input.links:
                # 获取连接的来源节点
                from_node = input.links[0].from_node
                # 递归处理来源节点
                self.find_connected_nodes(from_node, connected_nodes)
                
        return connected_nodes

    def draw_node_group_inputs(self, layout, node):
        """Draw input parameters for a node group"""
        if not node.node_tree:
            return
            
        # 检查节点是否有任何未连接的输入端口
        has_unconnected_inputs = False
        for input in node.inputs:
            if not input.links and not input.hide:
                # 检查是否有任何可调整的属性
                if (hasattr(input, "default_value") or 
                    hasattr(input, "default_value_raw") or
                    input.type in {'VECTOR', 'NORMAL', 'RGBA', 'BOOLEAN'}):
                    has_unconnected_inputs = True
                    break
                
        # 如果没有未连接的输入端口，不显示这个节点组
        if not has_unconnected_inputs:
            return
            
        # Create a box for the node group
        box = layout.box()
        row = box.row()
        row.label(text=node.name)
        row.prop(node, "mute", text="", icon='HIDE_OFF' if not node.mute else 'HIDE_ON', emboss=False)
        
        # Draw input parameters
        for input in node.inputs:
            # Skip connected or hidden inputs
            if input.links or input.hide:
                continue

            # Create row for each input
            row = box.row(align=True)
            
            # 处理不同类型的输入
            if input.type == 'BOOLEAN':
                # 布尔值输入
                if hasattr(input, "default_value"):
                    row.prop(input, "default_value", text=input.name)
                    
            elif input.type == 'VECTOR' or input.type == 'NORMAL':
                # 为矢量和法线类型创建一个子布局
                row.label(text=input.name)
                if hasattr(input, "default_value"):
                    sub = row.column(align=True)
                    sub.prop(input, "default_value", text="")
                elif hasattr(input, "default_value_raw"):
                    # 某些节点使用 default_value_raw
                    sub = row.column(align=True)
                    sub.prop(input, "default_value_raw", text="")
                
            elif input.type == 'RGBA':
                # 颜色输入
                row.label(text=input.name)
                if hasattr(input, "default_value"):
                    row.prop(input, "default_value", text="")
                
            elif hasattr(input, "default_value"):
                # 处理其他类型的输入
                row.label(text=input.name)
                
                if isinstance(input.default_value, float):
                    # 数值输入
                    row.prop(input, "default_value", text="")
                elif isinstance(input.default_value, (tuple, list)):
                    if len(input.default_value) == 3:  # Color
                        row.prop(input, "default_value", text="")
                    elif len(input.default_value) == 2:  # Vector2
                        split = row.split(factor=0.5)
                        split.prop(input, "default_value", text="", index=0)
                        split.prop(input, "default_value", text="", index=1)
                    elif len(input.default_value) == 3:  # Vector3
                        split = row.split(factor=0.33)
                        split.prop(input, "default_value", text="", index=0)
                        split = split.split(factor=0.5)
                        split.prop(input, "default_value", text="", index=1)
                        split.prop(input, "default_value", text="", index=2)

    def draw(self, context):
        layout = self.layout
        light = context.active_object.data
        
        # Light type
        row = layout.row()
        row.prop(light, "type", expand=True)

        # Common properties
        col = layout.column()
        col.prop(light, "energy")
        col.prop(light, "color")
        col.prop(light, "spread")
        
        # Specific properties based on light type
        if light.type == 'POINT':
            col.prop(light, "shadow_soft_size", text="Radius")
        elif light.type == 'SPOT':
            col.prop(light, "shadow_soft_size", text="Radius")
            col.prop(light, "spot_size")
            col.prop(light, "spot_blend")
        elif light.type == 'SUN':
            col.prop(light, "angle")
        elif light.type == 'AREA':
            col.prop(light, "shape")
            sub = col.column(align=True)
            if light.shape in {'SQUARE', 'DISK'}:
                sub.prop(light, "size")
            else:
                sub.prop(light, "size", text="Size X")
                sub.prop(light, "size_y", text="Size Y")
        
        # Node groups section
        if light.use_nodes:
            # 找到 Light Output 节点
            output_node = None
            for node in light.node_tree.nodes:
                if node.type == 'OUTPUT_LIGHT':
                    output_node = node
                    break
                    
            if output_node:
                # 获取所有与输出节点相连的节点
                connected_nodes = self.find_connected_nodes(output_node)
                
                # 检查是否有节点组需要显示
                has_node_groups = False
                for node in connected_nodes:
                    if node.type == 'GROUP':
                        # 检查是否有未连接的输入端口
                        for input in node.inputs:
                            if not input.links and hasattr(input, "default_value") and not input.hide:
                                has_node_groups = True
                                break
                        if has_node_groups:
                            break
                
                if has_node_groups:
                    layout.separator()
                    layout.label(text="Node Groups")
                    
                    # 只显示相连节点中的节点组
                    for node in connected_nodes:
                        if node.type == 'GROUP':
                            self.draw_node_group_inputs(layout, node)

class LIGHTING_GADGETS_OT_QUICK_PANEL(Operator):
    bl_idname = "lighting_gadgets.quick_panel"
    bl_label = "Light Settings"

    def invoke(self, context, event):
        if not context.active_object or context.active_object.type != 'LIGHT':
            self.report({'ERROR'}, "Please select a light object")
            return {'CANCELLED'}

        bpy.ops.wm.call_panel(name="LIGHTING_GADGETS_PT_quick_panel")
        return {'FINISHED'}